//   ConnectionManager 6 - Simple GUI app for Gnome 3 that provides a menu
//   for initiating SSH/Telnet/Custom Apps connections.
//   Copyright (C) 2011  Stefano Ciancio
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

const St = imports.gi.St;
const Gio = imports.gi.Gio;

const Shell = imports.gi.Shell;
const Util = imports.misc.util;
const Lang = imports.lang;
const Me = imports.misc.extensionUtils.getCurrentExtension();

const Config = imports.misc.config;

// SSH / Apps Search Provider
var ConnManSearchProvider = new Lang.Class({
    Name: 'ConnManSearchProvider',

    _init: function(title) {

        this.id = title;
        this.appInfo = Gio.DesktopAppInfo.new('org.gnome.Terminal.desktop');
        this.appInfo.get_name = function() { return _('Connection Manager'); };
        this.title = title;
        this.sshNames = [];

        //global.log('@@@CM 02a ConnManSearchProvider _init() title : ' + title + ' @@@');
        //global.log('@@@CM 02b ConnManSearchProvider _init() appInfo : ' + this._dumpObject(this.appInfo) + ' @@@');

    },

    // Update list of SSH/Apps on configuration changes
    _update: function (sshNames) {
        this.sshNames = sshNames;
    },


    filterResults: function(providerResults, maxResults) {
        //global.log('@@@CM 05 ConnManSearchProvider filterResults() : ' + this._dumpObject(providerResults) + ' @@@');
        return providerResults;
    },

    createResultObject: function(result, terms) {
        //global.log('@@@CM 05 ConnManSearchProvider createResultObject() : ' + this._dumpObject(result) + ' @@@');
        return null;
    },

    getInitialResultSet: function(terms, callback) {
        let searching = [];

        for (var i=0; i<this.sshNames.length; i++) {
            for (var j=0; j<terms.length; j++) {
                let pattern = new RegExp(terms[j],"gi");
                if (this.sshNames[i].name.match(pattern)) {
                    // +1 because id 0 breaks search results
                    searching.push(i+1);
                }
            }
        }

        //global.log('@@@CM 05 ConnManSearchProvider getInitialResultSet() : ' + this._dumpObject(searching) + ' @@@');

        if (typeof callback === "function") {
        //global.log('@@@CM 05 ConnManSearchProvider getInitialResultSet() : ' + 'CALLBACK @@@');
            callback(searching);
        }
    },

    getSubsearchResultSet: function(previousResults, terms, callback) {
        //global.log('@@@CM 05 ConnManSearchProvider getSubsearchResultSet() : ' + this._dumpObject(previousResults) + ' @@@');
        this.getInitialResultSet(terms, callback);
        //global.log('@@@CM 05 ConnManSearchProvider getSubsearchResultSet() : ' + ' END @@@');
    },

    getResultMetas: function(resultIds, callback) {
        let metas = [];
        let app = null;

        for (let i=0; i<resultIds.length; i++) {
            let result = this.sshNames[resultIds[i]-1]
            let appSys = Shell.AppSystem.get_default();
            let app = null;

            switch (result.type) {
                case '__app__':
                    app = null;
                    break;
                case '__item__':
                    app = appSys.lookup_app(result.terminal + '.desktop');
                    break;
            }

            metas.push({
                'id': resultIds[i],
                'name': result.name,
                'description': result.host,
                'createIcon': function(size) {
                    let icon = null;
                    let appt = app;

                    if (app)
                        icon = app.create_icon_texture(size);
                    else
                        icon = new St.Icon({ gicon: Gio.icon_new_for_string(Me.path + '/emblem-cm-symbolic.svg'),
                                             icon_size: size });

                    return icon;
                }
            })
        }
        //global.log('@@@CM 05 ConnManSearchProvider getResultMetas() : ' + this._dumpObject(metas) + ' @@@');

        callback(metas);
    },

    activateResult: function(id) {
		//global.log('@@@CM 03 ConnManSearchProvider activateResult() id : ' + id + ' @@@');
        //global.log('@@@CM 04 ConnManSearchProvider activateResult() sshNames : ' + this._dumpObject(this.sshNames[id-1]) + ' @@@');
        Util.spawnCommandLine(this.sshNames[id-1].command);
    },

	// http://www.openjs.com/scripts/others/dump_function_php_print_r.php
    _dumpObject: function (arr,level) {
		var dumped_text = "";
		if(!level) level = 0;
		//The padding given at the beginning of the line.
		var level_padding = "";
		for(var j=0;j<level+1;j++) level_padding += "    ";

		if(typeof(arr) == 'object') { //Array/Hashes/Objects
			for(var item in arr) {
				var value = arr[item];

				if(typeof(value) == 'object') { //If it is an array,
					dumped_text += level_padding + "'" + item + "' ...\n";
					if (level <= 5) {
						dumped_text += this._dumpObject(value,level+1);
					}
				} else {
					dumped_text += level_padding + "'" + item + "' => \"" + value + "\"\n";
				}
			}
		} else { //Stings/Chars/Numbers etc.
			dumped_text = "===>"+arr+"<===("+typeof(arr)+")";
		}
		return dumped_text;
	}

});
